//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#ifndef ANNOTATED_OUTPUT_H
#define ANNOTATED_OUTPUT_H

#include "ParseOutput.h"

#include <string>
#include <fstream>

class Namespace;

class AnnotatedOutput : public ParseOutput
{
 public:
    AnnotatedOutput(std::ofstream& stream, Namespace& ns,
		    const char *outputFile);
    ~AnnotatedOutput();
    virtual void start();
    virtual void keywordToken(const std::string& t);
    virtual void builtinTypeToken(const std::string& t);
    virtual void objectToken(const std::string& t, Symbol *symbol);
    virtual void classToken(const Class *c, const std::string& t);
    virtual void stringToken(const std::string& t);
    virtual void whitespaceToken(const std::string& t);
    virtual void commentToken(const std::string& t);
    virtual void otherToken(const std::string& t);
    virtual void operatorToken(const std::string& t);
    virtual void functionDefinition(const std::string& className,
                                    const std::string& functionName);
    virtual void functionCall(const std::string& caller,
                              const std::string& callee, bool callThru,
			      const std::string& t);
    virtual void objectDefinition(const std::string& className,
                                  const std::string& objectName);

 private:
    Namespace& _namespace;
    std::string _filename;
    std::ofstream& _out;
    bool _needsLineNo;
    int _lineNo;

    void printLineNo(int n);
};

#endif // ANNOTATED_OUTPUT_H
