//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#include "CallTreeOutput.h"
#include "Namespace.h"
#include "Function.h"

#include <assert.h>

using namespace std;

void CallTreeOutput::functionCall(const string& caller,
				  const string& callee, bool callThru,
				  const string& t)
{
    string::size_type funcBegin = caller.rfind('.');
    string callerClass = caller.substr(0, funcBegin);
    string callerName = caller.substr(funcBegin + 1);
    funcBegin = callee.rfind('.');
    string calleeClass = callee.substr(0, funcBegin);
    string calleeName = callee.substr(funcBegin + 1);
    Name actualClass;
    Member *mCaller = NULL, *mCallee = NULL;

    _namespace.lookup(callerClass, callerName, Symbol::kFunction,
		      &actualClass, &mCaller);
    _namespace.lookup(calleeClass, calleeName, Symbol::kFunction,
		      &actualClass, &mCallee);

    if (mCaller && mCallee) {
	assert(mCaller->symbolType() == Member::kFunction);
// Comment out this assert because we don't handle functions and methods
// with the same name yet.
//	assert(mCallee->symbolType() == Member::kFunction);
	if (mCallee->symbolType() != Member::kFunction)
	    return;
	assert(mCaller->seenDefinition());
	_callers[((Function *)mCallee)->getAncestor(_namespace)].insert(((Function *)mCaller)->getAncestor(_namespace));
    }
}

void CallTreeOutput::writeCallGraph(std::ofstream& out, Function *f) const
{
    // TODO: Be sure to avoid infinite recursion if a function calls itself

    CallerMap::const_iterator it = _callers.find(f);
    if (it == _callers.end())
	return;

    out << "digraph callgraph {" << endl;
    out << "   graph [bgcolor=transparent,rankdir=RL];" << endl;
    out << "   node [shape=box,fontsize=10,fontname=Arial,height=.25,style=filled,fillcolor=white];" << endl;

    CallerSet::const_iterator cit;
    string text;
    for (cit = it->second.begin();  cit != it->second.end();  ++cit) {
	text.clear();
	if ((*cit)->owningClass() != f->owningClass())
	    text += (*cit)->owningClass()->shortName() + string(".\\n");
	text += (*cit)->name();
	out << "   \"" << text << "\"->\"";
	out << f->name() << "\"" << endl;
	out << "   \"" << text << "\" [URL=\"javascript:load('"
	    << (*cit)->owningClass()->classFilename() << "', '"
	    << (*cit)->definitionHref() << "', '"
	    << (*cit)->getAncestor(_namespace)->callgraphFilename() << "')\"]"
	    << endl;
    }
    out << "}" << endl;
}

std::string CallTreeOutput::getString(Function *f)
{
    return f->owningClass()->name() + f->mangledName();
}
