//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#ifndef FUNCTION_H
#define FUNCTION_H

#include "Member.h"
#include "Class.h"

#include <vector>

class Namespace;

class Function : public Member
{
 public:
    struct Arg
    {
	enum Flags { kConst = 1<<1, kFinal = 1<<2 };
	Name name;
	Name type;
	int flags;
	std::string value;  // default value (or empty string otherwise)

	Arg() {};  // So we can put this in STL containers; don't use
	Arg(const Name& name_, const Name& type_, int flags_ = false,
	    const std::string& value_ = "")
	    : name(name_), type(type_), flags(flags_), value(value_)
	    {}
    };
    typedef std::vector<Arg> Args;

    Function() : Member() {}
    Function(Class *c, const Name& name, const Name& type, const Args& args,
	     int protection = Member::kPrivate)
	: Member(c, name, type, protection)
	{
	    Args::const_iterator it;
	    for (it = args.begin();  it != args.end();  ++it)
		_args.push_back(Arg(it->name, it->type, it->flags, it->value));
	}

    virtual SymbolType symbolType() const { return kFunction; }

    virtual std::string mangledName() const;

    virtual std::string anchor() const;

    const Args& args() { return _args; }
    void addArgs(const Arg& a ) { _args.push_back(a); }

    /** Returns the highest-level virtual function that this function overrides.
	If the function is not virtual, returns itself. */
    Function* getAncestor(Namespace& ns);

    /** Note:  this may return a name that must be quoted if passed to a
               shell command */
    std::string callgraphFilename() const {
	return owningClass()->name() + std::string(".") + mangledName() +
	    			       std::string("-callers.html");
    }
    /** Note:  this may return a name that must be quoted if passed to a
               shell command */
    std::string callgraphImageFilename() const {
	return owningClass()->name() + std::string(".") + mangledName() +
	    			       std::string("-callers.png");
    }
    /** Note:  this may return a name that must be quoted if passed to a
               shell command */
    std::string callgraphImageMapFilename() const {
	return owningClass()->name() + std::string(".") + mangledName() +
	    			       std::string("-callers.map");
    }
    /** Note:  this may return a name that must be quoted if passed to a
               shell command */
    std::string callgraphDotFilename() const {
	return owningClass()->name() + std::string(".") + mangledName() +
	    			       std::string("-callers.dot");
    }

 private:
    Args _args;
};

#endif // FUNCTION_H
