//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#include "Global.h"
#include "Class.h"
#include "Member.h"
#include "Function.h"
#include "Namespace.h"

Settings gSettings;
ParseState gParseState;

std::string html(const std::string& str)
{
    std::string s = str;
    std::string::size_type pos = 0;

    // Replace ampersands first because these substitutions add them!
    while ((pos = s.find('&', pos + 1)) != std::string::npos)
	s.replace(pos, 1, "&amp;");

    pos = 0;
    while ((pos = s.find('<', pos)) != std::string::npos)
	s.replace(pos, 1, "&lt;");

    pos = 0;
    while ((pos = s.find('>', pos)) != std::string::npos)
	s.replace(pos, 1, "&gt;");

    return s;
}

std::string makePrettyName(const std::string& name)
{
    std::string s;
    for (unsigned int i = 0;  i < name.size();  ++i) {
	if (name[i] == '.')
	    s += ".\\n";
	else
	    s.push_back(name[i]);
    }

    return s;
}

std::string getClassLink(const Class& c)
{
    // Convert typename to html because it might be templated
    return getClassLink(c, html(c.shortName()));
}

std::string getClassLink(const Class& c, const std::string& htmlText)
{
    if (!c.seenDefinition())
	return htmlText;

    std::string link = "<a href=\"javascript:load('";
    link += c.classFilename();
    link += "', '";
    link += c.sourceFilename();
    link += "', '')\">";
    link += htmlText;
    link += "</a>";

    return link;
}

std::string getMemberLink(Namespace& ns, const Member *m, bool switchClass)
{
    return getMemberLink(ns, m, switchClass, m->name());
}

std::string getMemberLink(Namespace& ns, const Member *m, bool switchClass,
			  const std::string& htmlText,
			  const std::string& cssClass /*= ""*/)
{
    std::string link = "<a ";
    if (!cssClass.empty())
	link += std::string("class=\"") + cssClass + std::string("\" ");
    link += "href=\"javascript:load('";
    if (switchClass)
	link += m->owningClass()->classFilename();
    link += "', '";
    link += m->definitionHref();
    link += "', '";
    if (m->symbolType() == Symbol::kFunction)
	link += ((Function *)m)->getAncestor(ns)->callgraphFilename();
    link += "')\">";
    link += htmlText;
    link += "</a>";

    return link;
}

