//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#ifndef GRAPH_H
#define GRAPH_H

#include "Global.h"
#include "Counter.h"
#include "Namespace.h"

#include <map>

class Graph {
 private:
    struct Node {
	Node() : name("Internal error:  Graph::Node::Node() should never be called!") {}
	Node(const std::string& name_) : name(name_) {}
	int edges() { return in.size() + out.size(); }
	bool onlyInOrOut() { return (in.empty() || out.empty()); }

	std::string name;
	std::map<Name, Counter> in;
	std::map<Name, Counter> out;

	void print() const;
    };
    std::map<Name, Node> _nodes;

 public:
    Graph(Namespace& ns) : _namespace(ns) {};

    // Adds a caller->callee relationship, unless (caller instanceof callee)
    // or callee's object has not been defined.  Obviously you will want
    // to process all the class definitions you are interested in before
    // calling this!
    void add(const std::string& caller, const std::string& callee,
	     const std::string& mods);

    void getObject(const std::string& call,
		   std::string *obj, std::string *method);
    void reduceToCycles();

    void print();
    void printClasses();

 private:
    Namespace& _namespace;
    std::map<std::string, int> _externalCalls;
    std::set<std::string> _usedMethods;
};

extern Graph gGraph;

#endif // GRAPH_H
