//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#ifndef JAVA_PARSER_H
#define JAVA_PARSER_H

#include "Parser.h"
#include "ParseOutput.h"

#include <string>

class ScopedNames;

class JavaParser : public Parser
{
 public:
    JavaParser(Namespace& ns, ParseOutput& o, const char *filename)
	: Parser(ns, o, filename)
	{};

 protected:
    virtual bool parseLine(ScopedNames& scope, TokenArray& tokens, Mode mode);
    virtual bool isObjDereference(const std::string& token);
    virtual bool isBuiltinType(const std::string& token);

 private:
    enum State { kEOL = -1, kUnimportant = 0, kClassDef, kVariableDef,
		 kMethodDef, kObjDeref, kOpenBlock, kCloseBlock, kAnnotation,
		 kScopeModifier, kPackage, kImport };

    unsigned int processFunctionArgs(TokenArray& tokens, unsigned int idx,
				     ScopedNames& scope, Function::Args *args);
    State nextState(TokenArray& tokens, int idx);
};

#endif // JAVA_PARSER_H
