//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#ifndef MEMBER_H
#define MEMBER_H

#include "Symbol.h"

#include <string>

class Class;

class Member : public Symbol
{
 public:
    enum Modifier{ kPrivate =		1<<0,
		   kProtected =		1<<1,
		   kPublic =		1<<2,
		   kProtectionMask =	( kPrivate | kProtected | kPublic ),

		   kStatic =		1<<3,
		   kVirtual =		1<<4,
		   kPureVirtual =	1<<5,
		   kFinal =		1<<6,
		   kConst =		1<<7};

    Member() : Symbol(), _modifiers(kPrivate) {}
    Member(Class *c, const Name& name, const Name& type,
	   int modifiers = kPrivate)
	: Symbol(name), _class(c), _type(type), _modifiers(modifiers)
	{}
    virtual ~Member() {};

    virtual SymbolType symbolType() const { return kMember; }

    const Name& type() const { return _type; }
    int modifiers() const { return _modifiers; }

    class Class* owningClass() const { return _class; }

    virtual std::string anchor() const;

    /** Returns a unique name.  Functions will be encoded with their arguments,
	but this is NOT the C++ name-mangling.  */
    virtual std::string mangledName() const;
    
    static int parseModifier(const std::string& token);

 private:
    Class *_class;
    Name _type;
    int _modifiers;
};

#endif // MEMBER_H
