//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#ifndef NAME_TABLE_H
#define NAME_TABLE_H

#include <string>
#include <vector>
#include <map>

class Symbol;

template <typename T>
class NameTable
{
 public:
    typedef typename std::multimap<std::string, T*> NameMap;
    typedef typename NameMap::const_iterator const_iterator;

    ~NameTable()
    {
	const_iterator it;
	for (it = _names.begin();  it != _names.end();  ++it)
	    delete it->second;
    }

    unsigned int count(const std::string& name) const
	{ return _names.count(name); }
    /** Inserts a (name, value) pair into the name table.  The table
	owns the Symbol pointer. */
    void insert(const std::string& name, T *value) {
	_names.insert(std::pair<std::string, T*>(name, value));
    }
    unsigned int find(const std::string& name,
		      std::vector<T *> *values) const
    {
	std::pair<const_iterator, const_iterator> range;
	const_iterator it;
    
	range = _names.equal_range(name);
	values->clear();
	for (it = range.first;  it != range.second;  ++it)
	    values->push_back(it->second);

	return values->size();
    }

    const_iterator begin() const { return _names.begin(); }
    const_iterator end() const { return _names.end(); }

 private:
    NameMap _names;
};

#endif // NAME_TABLE_H
