//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#include "Namespace.h"

#include <iostream>
#include <vector>

using namespace std;

bool Namespace::lookup(const Name& className, const Name& name, 
		       Symbol::SymbolType requestedType,
		       Name *actualClass, Member **member) const
{
//    cerr << "Namespace::lookup(" << className << ", " << name << ")" << endl;
    Name untemplatizedName = className.substr(0, className.find('<'));
//    cout << "   untemplatizedName: '" << untemplatizedName << "'" << endl;
    ClassMap::const_iterator it = _classes.find(untemplatizedName);
    if (it == _classes.end()) {
//	cerr << "   Can't find class" << endl;
	return false;
    }
    vector<Member *> mm;
    if (it->second.members_const().find(name, &mm) == 0) {
	if (it->second.parents().empty()) { // can't be inherited
	    if (it->second.isInnerClass()) {
		return lookup(it->second.outerClass(), name, requestedType,
			      actualClass, member);
	    }
//	    cerr << "   No parents, can't find name in class" << endl;
	    return false;
	}
	bool retVal = false;
	vector<Name>::const_iterator p = it->second.parents().begin();
	for (; !retVal && p != it->second.parents().end();  ++p)
	    retVal = lookup(*p, name, requestedType, actualClass, member);
	return retVal;
    }

    // Because you can overload functions and variables with the same name
    // (at least in Java), find the one that you are looking for
    unsigned int idx = 0;
    while (idx < mm.size() && mm[idx]->symbolType() != requestedType)
	idx++;

    if (idx >= mm.size())
	return false;

    *actualClass = className;
    *member = mm[idx];
    return true;
}

bool Namespace::instanceOf(const Name& name, const Name& classInQuestion)
{
    ClassMap::const_iterator obj = _classes.find(name);
    ClassMap::const_iterator potentialClass = _classes.find(classInQuestion);
    
    if (obj == _classes.end() || potentialClass == _classes.end())
	return false;
    if (obj->first == potentialClass->first)
	    return true;
    if (obj->second.parents().empty())
	return false;

    bool retVal = false;
    vector<Name>::const_iterator p = obj->second.parents().begin();
    for (;  !retVal && p != obj->second.parents().end();  ++p)
	retVal = instanceOf(*p, classInQuestion);
    return retVal;
}

Name Namespace::getInheritedMethod(const Name& className, const Name& method)
{
    Name actualClass;
    Member *unused;
    bool exists = lookup(className, method, Symbol::kFunction,
			 &actualClass, &unused);
    
    if (exists)
	return actualClass + string(".") + method;
	
    // If not here, then assume that this is inherited from a base
    // class we don't know about.
//    const vector<Name> &parents = get(className).parents();
//    if (parent != "")
//	return parent + string(".") + method;

    // Ok, now we just have no idea, so just leave it as it is.
    return className + string(".") + method;
}

void Namespace::printSubGraphs() const
{
/*
    ClassMap::const_iterator it;
    for (it = _classes.begin();  it != _classes.end();  ++it) {
	if (it->second.members_const().nUsedMembers() > 1) {
	    cout << "   subgraph \"" << it->first << "\" {" << endl;
	    cout << "      label=\"" << it->first << "\"" << endl;
	    it->second.members_const().printDot(it->first);
	    cout << "   }" << endl;
	}
    }
*/
    cerr << "Namespace::printSubGraphs() is not implemented!" << endl;
}

void Namespace::printHierarchy(std::set<Name> *used) const
{
/*
    ClassMap::const_iterator it;
    for (it = _classes.begin();  it != _classes.end();  ++it) {
	if (it->second.seenDefinition() && !it->second.parents().empty()) {
	    bool usedParent = false;
	    const vector<Name>& parents = it->second.parents();
	    for (unsigned int i = 0;  i < parents.size() && !usedParent;  ++i)
		if (used->find(parents[i]) != used->end())
		    usedParent = true;
	    if (used && !usedParent)
		continue;
	    // Using parents[0] is wrong!
	    cout << "   \"" << it->first << "\"->\"" << parents[0]
		 << "\" [style=bold,weight=8];" << endl;
	}
    }
*/
    cerr << "Namespace::printHierarchy() is not implemented!" << endl;
}

void Namespace::print() const
{
    ClassMap::const_iterator it;
    cerr << "--------- Namespace table --------------" << endl;
    for (it = _classes.begin();  it != _classes.end();  ++it) {
	cerr << "class " << it->first;
	const vector<Name>& parents = it->second.parents();
	if (!parents.empty()) {
	    cerr << " extends ";
	    for (unsigned int i = 0;  i < parents.size();  ++i) {
		if (i > 0)
		    cerr << ", ";
		cerr << parents[i];
	    }
	}
	cerr << endl;
//	it->second.members_const().print();
	Class::Members::const_iterator it2 = it->second.members_const().begin();
	for (;  it2 != it->second.members_const().end();  ++it2)
	    cerr << "   " << it2->first << " -> " << it2->second << endl;
    }
    cerr << "----------------------------------------" << endl;
}
