//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#ifndef NAMESPACE_H
#define NAMESPACE_H

#include "Global.h"
#include "Class.h"

#include <string>
#include <map>
#include <set>

class Namespace
{
 public:
    typedef std::map<Name, Class> ClassMap;

    Class& get(const Name& name) {
	Class& c = _classes[name];
	if (c.name().empty())
	    c.setName(name);
	return c;
    }
    bool has(const Name& name)
	{ return (_classes.find(name) != _classes.end()); }
    bool lookup(const Name& className, const Name& name, 
		Symbol::SymbolType requestedType,
		Name *actualClass, Member **member) const;
    bool instanceOf(const Name& name, const Name& classInQuestion);
    Name getInheritedMethod(const Name& className, const Name& method);

    void printSubGraphs() const;
    void printHierarchy(std::set<Name> *used) const;
    void print() const;

    class iterator
    {
     public:
	iterator() {}
	iterator(ClassMap::iterator it) : _it(it) {}
	Class& operator*() { return _it->second; }
	Class* operator->() { return &_it->second; }
	bool operator!=(const iterator& that) { return this->_it != that._it; }
	iterator operator++() { _it++;  return *this; }
	iterator& operator++(int) { _it++;  return *this; }
	const Name& key() { return _it->first; }
     private:
	ClassMap::iterator _it;
    };
    iterator begin() { return _classes.begin(); }
    iterator end() { return _classes.end(); }

 private:
    ClassMap _classes;
};

#endif // NAMESPACE_H
