//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#ifndef PARSER_H
#define PARSER_H

#include "Global.h"
#include "Namespace.h"
#include "ScopedNames.h"
#include "Token.h"
#include "ParseOutput.h"
#include "Function.h"

#include <vector>
#include <string>

class Parser {
 public:
    enum Mode { kBuildHierarchy, kOutputCallGraph };

    Parser(Namespace& ns, ParseOutput& o, const char *filename)
	: _namespace(ns), _output(o), _filename(filename)
	{};
    virtual ~Parser() {};
    bool parse(Mode mode);

 protected:
    enum NameType { kVariable, kMethod };

    struct NameLookupInfo {
	std::string name;
	bool isClass;

	NameLookupInfo(const std::string& n, bool isC)
	    : name(n), isClass(isC)
	    {};
    };

    typedef std::vector<Token> TokenArray;
    typedef std::vector<NameLookupInfo> NameResolutionList;
    Namespace& _namespace;
    ParseOutput& _output;
    NameResolutionList _nameResolutionList;

    virtual bool parseLine(ScopedNames& scope, TokenArray& tokens, Mode mode)=0;
    virtual bool isObjDereference(const std::string& token) = 0;
    virtual bool isBuiltinType(const std::string& token) = 0;

    void parseExpression(TokenArray& tokens, unsigned int idx,
			 ScopedNames& scope);

    bool setClassToken(Token& token, const Name& className, ScopedNames& scope);
    void addName(ScopedNames& scope, NameType type,
		 const Name& templatedName, const Name& retType, int modifiers,
		 const Function::Args& args = Function::Args());

    bool resolveClass(const ScopedNames& scope, const Name& templatedClassName,
		      Name *resolvedName);
    std::string untemplatedName(const Name& templatedName);

    int joinTokens(TokenArray& tokens, unsigned int startIdx,
		   const std::string& joinStr, std::string *joined);

    bool nextLine(std::string *line);
    void appendNextLine(TokenArray *tokens);
    unsigned int nextToken(TokenArray& tokens, unsigned int idx,
			   unsigned int n = 1);
    unsigned int prevToken(TokenArray& tokens, unsigned int idx,
			   unsigned int n = 1);
    TokenArray getTokens(const std::string& line);
    unsigned int findMatchingParen(TokenArray tokens, unsigned int startIdx);

    bool isName(const std::string& str)
	{ return (isAlphaNumeric(str[0]) &&
		  !(str[0] >= '0' && str[0] <= '9')); }
    bool isWhitespace(const char c)
	{ return (c == ' ' || c == '\t' || c == '\n' || c == '\r'); }
    bool isAlphaNumeric(const char c)
	{ return ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') ||
		  (c >= '0' && c <= '9') || c == '_'); }
    void printTokenArray(const TokenArray& tokens);

 private:
    std::string _filename;
    FILE *_in;
};

#endif // PARSER_H
