//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#ifndef SCOPED_NAMES_H
#define SCOPED_NAMES_H

#include "Global.h"
#include "Name2Type.h"
#include "Namespace.h"

#include <vector>

class ScopedNames {
 public:
    ScopedNames(Namespace& ns) : _namespace(ns) {}
    enum ScopeType { kBlock, kClass, kMethod };

    const Name& package() const { return _package; }
    void setPackage(const Name &name) { _package = name; }

    void add(const Name& name, const Name& type)
	{ _scopes.back().add(name, type); }

    void pushBlock();
    void pushClass(const std::string& name);
    void pushMethod(const std::string& name);
    void popBlock();
    Name find(const Name& name) const;
    int depth() { return _names.size(); }
    Name getClass() const;
    Name getMethod() const;
    Name getScopedName(const Name& name) const;

    bool inMethod() const;
    /** Returns true and sets 'type' to the type of the variable if the
	scope is within a method and if the variable exists.  Otherwise
	returns false and sets 'type' to "" */
//    bool lookupVariable(const Name& name, Name *type) const;

 private:
    Namespace& _namespace;
    std::vector<Name2Type> _scopes;
    std::vector<Name> _names;
    std::vector<ScopeType> _types;
    Name _package;
};

#endif // SCOPED_NAMES_H
