//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#ifndef SYMBOL_H
#define SYMBOL_H

#include "Global.h"

#include <string>

class Symbol
{
 public:
    enum SymbolType { kSymbol, kClass, kMember, kFunction };

    Symbol() : _definedAtLine(-1) {}
    Symbol(const Name& name)
	: _name(name)
	{}
    virtual ~Symbol() {}

    virtual SymbolType symbolType() const { return kSymbol; }

    const Name& name() const { return _name; }
    /** Sets the name of the symbol. Asserts if the symbol already has a name */
    virtual void setName(const Name& name);

    void setDefinition(const std::string& file, int line);
    const std::string& definitionFile() const{ return _definedInFile; }
    int definitionLine() const { return _definedAtLine; }
    
    bool seenDefinition() const { return _definedAtLine > 0; }

    virtual std::string anchor() const { return ""; }
    std::string definitionHref() const;
    const std::string sourceFilename() const{ return _sourceFile + std::string(".html"); }

 private:
    Name _name;
    std::string _definedInFile;
    int _definedAtLine;
    std::string _sourceFile;
};

#endif // SYMBOL_H
