//-------------------- Copyright (C) 2009 by Geoffrey Prewett -----------------
#ifndef TOKEN_H
#define TOKEN_H

#include <assert.h>
#include <string>

// TODO:  This class really needs to be re-architected!!

class Token
{
 public:
    enum Type { kUnknown = -1, kNothing = 0, kObject = 1, kOperator,
		kBuiltinType, kClass, kFunctionCall, kKeyword, kString,
		kConstant, kFunctionDefinition, kObjectDefinition,
		kWhitespace, kComment };
    Token(const std::string& s, Type t)
	: _token(s), _type(t), _callThru(false)
	{};

    const std::string& str() const { return _token; }
    void setStr(const std::string& s) { _token = s; }
    void appendToken(const std::string& str) { _token += str; }
    Type type() const { return _type; }
    void setType(Type t) { assert(_type == kUnknown || _type == t); _type = t; }

    void setFunctionCall(const std::string& caller,
			 const std::string& callee, bool callThru)
    {
	_caller = caller;
	_callee = callee;
	_callThru = callThru;
	_type = kFunctionCall;
    }
    const std::string& caller() const { return _caller; }
    const std::string& callee() const { return _callee; }
    bool callThru() const { return _callThru; }

    void setDefinition(Type type, const std::string& scopeName)
    {
	_type = type;
	_defScope = scopeName;
    }
    const std::string& defScope() const { return _defScope; }

    void setClassRef(const std::string& fullName)
    {
	_type = kClass;
	_defScope = fullName;
    }
    const std::string& fullClassName() const { return _defScope; }

    bool operator==(const std::string& str) const
        { return (_token == str); }
    bool operator!=(const std::string& str) const
        { return (_token != str); }

 private:
    std::string _token;
    Type _type;
    std::string _caller;  // Valid for kFunctionCall
    std::string _callee;  // Valid for kFunctionCall
    bool _callThru;       // Valid for kFunctionCall
    std::string _defScope;
};

#endif // TOKEN_H
